/* **************************************************************** *
 *                                                                  *
 *  APRSG-NG -- 2nd generation receive-only APRS-i-gate with        *
 *              minimal requirement of esoteric facilities or       *
 *              libraries of any kind beyond UNIX system libc.      *
 *                                                                  *
 * (c) Matti Aarnio - OH2MQK,  2007                                 *
 *                                                                  *
 * **************************************************************** */

#include "aprsg.h"

/* Bits used only in the main program.. */
#include <signal.h>

time_t now; /* this is globally used */
int debug;
int verbout;

#define CFGFILE "/etc/aprsg-ng.conf"

#define MAXPOLLS 24 /* No really all that much, 16 serial ports
		       4 network socket to APRS-IS, something
		       for latter inventions.. */

struct pollfd polls[MAXPOLLS];

int die_now;

const char *version = "aprsg-ng-v0.04";


static void sig_handler(int sig)
{
  die_now = 1;
}

static void usage(void)
{
	printf("aprsg-ng: [-d] [-f %s]\n", CFGFILE);
	printf("    -d:  turn debug printout on, use to verify config file!\n");
	printf("    -f %s:  where the configuration is\n", CFGFILE);
	exit(64); /* EX_USAGE */
}


int main(int argc, char * const argv[]) 
{
	int i;
	struct pollfd *fds;
	int nfds, nfds2;
	const char *cfgfile = "/etc/aprsg-ng.conf";

	now = time(NULL);

	setlinebuf(stdout);
	setlinebuf(stderr);

	/* Init the poll(2) descriptor array */
	memset(polls, 0, sizeof(polls));
	for (i = 0; i < MAXPOLLS; ++i)
	  polls[i].fd = -1;

	signal(SIGTERM, sig_handler);
	signal(SIGINT, sig_handler);

	while ((i = getopt(argc, argv, "df:hv?")) != -1) {
	  switch (i) {
	  case '?': case 'h':
	    usage();
	    break;
	  case 'd':
	    ++debug;
	    break;
	  case 'v':
	    ++verbout;
	    break;
	  case 'f':
	    cfgfile = optarg;
	    break;
	  default:
	    break;
	  }
	}


	ttyreader_init();
	aprsis_init();
	netax25_init();

	readconfig(cfgfile); /* TODO: real parametrized cfg file location.. */



	/* The main loop */

	while (! die_now ) {

	  time_t next_timeout;
	  now = time(NULL);
	  next_timeout = now + 30;

	  aprsis_cond_reconnect();

	  fds = polls;
	  nfds = MAXPOLLS;

	  for (i = 0; i < MAXPOLLS; ++i)   polls[i].fd = -1;

	  i = ttyreader_prepoll(nfds, &fds, &next_timeout);
	  nfds -= i;
	  nfds2 = i;
	  i = aprsis_prepoll(nfds, &fds, &next_timeout);
	  nfds2 += i;
	  nfds  -= i;
	  i = beacon_prepoll(nfds, &fds, &next_timeout);
	  nfds2 += i;
	  nfds  -= i;
	  i = netax25_prepoll(nfds, &fds, &next_timeout);
	  nfds2 += i;
	  nfds  -= i;

	  if (next_timeout <= now)
	    next_timeout = now + 1; /* Just to be on safe side.. */

	  i = poll(polls, nfds2, (next_timeout - now) * 1000);
	  now = time(NULL);


	  i = beacon_postpoll(nfds2, polls);
	  i = ttyreader_postpoll(nfds2, polls);
	  i = netax25_postpoll(nfds2, polls);
	  i = aprsis_postpoll(nfds2, polls);

	}

	exit (0);
}

